/*
 * Copyright (c) 2012 Alexandre Tuleu
 *
 * This file is part of xeno-cpg.
 *
 *   xeno-cpg is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   xeno-cpg is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with PROJECT.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * \file CpgNetworkRTWrapper.h
 *
 *  \date Jul 1, 2011
 *  \author Alexandre Tuleu
 */

#ifndef XENO_CPG_CPG_NETWORK_RT_WRAPPER_H_
#define XENO_CPG_CPG_NETWORK_RT_WRAPPER_H_

#include <map>
#include <tr1/memory>
#include <vector>
#include <string>

#include <cpg-network/cpg-network.h>

#include <libatuleu/options/Option.h>

class CpgIOBase {
public :
	CpgIOBase();
	virtual ~CpgIOBase();

	double Value() const;
	void SetValue(double value);
private :
	double d_value;
};

class CpgOutput : public CpgIOBase {
public :
	typedef std::tr1::shared_ptr<CpgOutput> Ptr;

	CpgOutput();
	virtual ~CpgOutput();

	virtual void Send() = 0;

};

class CpgInput : public CpgIOBase {
public :
	typedef std::tr1::shared_ptr<CpgInput> Ptr;

	CpgInput();
	virtual ~CpgInput();
	virtual void Read() = 0;
};





class CpgPropertyIO {
public :
	typedef std::tr1::shared_ptr<CpgPropertyIO> Ptr;

	CpgPropertyIO(CpgProperty * property, const std::string & name);
	virtual ~CpgPropertyIO();

	void Set(double value);
	double Read() const;

	const std::string & Name() const;
private :
	CpgProperty * d_property;
	const std::string d_name;
};


/**
 *
 */
class CpgNetworkRTWrapper {
public:
	typedef std::vector<CpgPropertyIO::Ptr> CpgPropertyIOList;
	typedef double Input;

	static void InitRtAccess();
	static void WaitForRTAccessInitialized();

	CpgNetworkRTWrapper();
	virtual ~CpgNetworkRTWrapper();

	/**
	 * \returns \true if everything is ok.
	 */
	bool operator() ()const;

	int Step();

	void Start();
	void Pause();

	void Compile();
	void Init();

	const CpgPropertyIOList & GlobalInputProperty();

	//CpgPropertyIO::Ptr FromSerialized(const std::string & serialized);

	//bool LinkPropToValuePointer(const std::string & serializedName,
	//const double & valueHolder);
	//bool UploadValuesToCPG();
protected :

private :
	typedef std::tr1::shared_ptr<Input>  InputPtr;
	typedef std::map<CpgProperty*,CpgOutput::Ptr> OutputPropertyMapper;
	typedef std::map<CpgProperty*,CpgInput::Ptr>  InputPropertyMapper;
	
	static const char * s_mutexName;

	void InitMappingToServo();
	void InitNetworkOverride();
	void InitGlobals();
	void DeInit();
	bool Valid();
	CpgProperty * FindPropertyFromSerialized(const std::string & serialized,
	                                         std::string * canonicalName = NULL);
	void UploadValuesToCPG();


	OutputPropertyMapper d_outputs;
	InputPropertyMapper  d_inputs;

	CpgPropertyIOList d_globals;

	bool d_valid;

	CpgNetwork * d_network;
	unsigned int d_iterationPerStep;
	double d_ts;
	double d_initTime;
	double d_time;
	bool d_paused;




};


#endif // XENO_CPG_CPGCONTROLLER_H_
