/*
 * Copyright (c) 2012 Alexandre Tuleu
 *
 * This file is part of xeno-cpg.
 *
 *   xeno-cpg is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   xeno-cpg is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with PROJECT.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * \file RTIO.cpp
 *
 *  \date Jun 14, 2012
 *  \author Alexandre Tuleu
 */

#include "RTIO.h"
#include <stdexcept>

RTIO::RTIO(){
}

RTIO::~RTIO(){
}

IOHook::~IOHook(){}

void IOHook::PreSend(){}

void IOHook::PostSend(){}

void IOHook::PreRead(){}

void IOHook::PostRead(){}

void IOHook::Print(std::ostream & ){}


void RTIO::RegisterOutput(const std::string& name , CpgOutput::Ptr out){
	OutputByName::const_iterator fi = d_outputs.find(name);
	if ( fi != d_outputs.end()){
		throw std::runtime_error("'" + name + "' is already registered.");
	}
	d_outputs.insert(std::make_pair(name,out));
}

void RTIO::RegisterInput(const std::string& name , CpgInput::Ptr in){
	InputByName::const_iterator fi = d_inputs.find("name");
	if( fi != d_inputs.end() ){
		throw std::runtime_error("'" + name + "' is already registered.");
	}
	d_inputs.insert(std::make_pair(name,in));
}

CpgOutput::Ptr RTIO::Output(const std::string& name){
	OutputByName::iterator fi = d_outputs.find(name);
	if( fi == d_outputs.end() ){
		return CpgOutput::Ptr();
	}
	return fi->second;
}

CpgInput::Ptr RTIO::Input(const std::string& name){
	InputByName::iterator fi = d_inputs.find(name);
	if( fi == d_inputs.end() ){
		return CpgInput::Ptr();
	}
	return fi->second;
}

#define CALL_HOOKS(hook)do{\
	for(ListOfHooks::iterator h = d_hooks.begin();\
	    h != d_hooks.end();\
	    ++h){\
		(*h)->hook();\
	}\
}while(0)


void RTIO::Sync(){
	CALL_HOOKS(PreSend);

	for(OutputByName::iterator o = d_outputs.begin();
	    o != d_outputs.end();
	    ++o){
		o->second->Send();
	}

	CALL_HOOKS(PostSend);

	CALL_HOOKS(PreRead);

	for(InputByName::iterator i = d_inputs.begin();
	    i != d_inputs.end();
	    ++i){
		i->second->Read();
	}
	CALL_HOOKS(PostRead);
}

void RTIO::Print(std::ostream & out){
	for(ListOfHooks::iterator h = d_hooks.begin();
	    h != d_hooks.end();
	    ++h) {
		(*h)->Print(out);
	}
}
