/* 
 * Copyright (c) 2012 Alexandre Tuleu
 * 
 * This file is part of xeno-cpg.
 * 
 *   xeno-cpg is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 *   xeno-cpg is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License
 *   along with PROJECT.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * \file RTIO.h
 *
 *  \date Jun 14, 2012
 *  \author Alexandre Tuleu
 */

#ifndef XENO_CPG_RTNODEIO_H_
#define XENO_CPG_RTNODEIO_H_

#include <tr1/memory>
#include <map>
#include <vector>
#include <iostream>

#include <libatuleu/base/Singleton.h>

#include "CpgNetworkRTWrapper.h"


class IOHook {
public :
	typedef std::tr1::shared_ptr<IOHook> Ptr;
	virtual ~IOHook();
	virtual void PreSend();
	virtual void PostSend();
	virtual void PreRead();
	virtual void PostRead();
	virtual void Print(std::ostream & out);
};

/**
 *
 */
class RTIO : public Singleton<RTIO>{
public:
	RTIO();
	virtual ~RTIO();

	void RegisterOutput(const std::string & name, CpgOutput::Ptr out);
	void RegisterInput(const std::string & name, CpgInput::Ptr in);

	CpgOutput::Ptr Output(const std::string & name);
	CpgInput::Ptr  Input (const std::string & name);

	void Sync();
	void AppendHook(IOHook::Ptr hook);

	void Print(std::ostream & out);


private :

	typedef std::map<std::string,CpgOutput::Ptr> OutputByName;
	typedef std::map<std::string,CpgInput::Ptr>  InputByName;
	typedef std::vector<IOHook::Ptr>               ListOfHooks;

	OutputByName d_outputs;
	InputByName  d_inputs;
	ListOfHooks  d_hooks;
};


#endif // XENO_CPG_RTNODEIO_H_
