/*
 * Copyright (c) 2012 Alexandre Tuleu
 *
 * This file is part of xeno-cpg.
 *
 *   xeno-cpg is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   xeno-cpg is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with PROJECT.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * \file XenoCpg.h
 *
 *  \date Jun 18, 2012
 *  \author Alexandre Tuleu
 */

#ifndef XENO_CPG_XENO_CPG_H_
#define XENO_CPG_XENO_CPG_H_

#include <sys/mman.h>

#include <tr1/memory>
#include <string>
#include <native/task.h>
#include <pthread.h>

#include <libatuleu/base/Singleton.h>

#include "CpgNetworkRTWrapper.h"

#define XENO_CPG_INIT()do{\
	mlockall(MCL_CURRENT|MCL_FUTURE);\
	/*force instantiation of singleton*/\
	XenoCpg::Instance();\
}while(0)



class XenoCpg : public Singleton<XenoCpg>{
public :
	XenoCpg();
	virtual ~XenoCpg();

	class Initializer {
	public:
		typedef std::tr1::shared_ptr<Initializer> Ptr;
		virtual void Init();
	};
	void Init(const std::string & name, int & argc, char ** argv);
	void StartMainLoop();
	void StartPrompt();
	void WaitForCompletion();

	void AppendInitializer(Initializer::Ptr init);
	template <class I> void AppendInitializer();
private :
	typedef std::vector<Initializer::Ptr> ListOfInitializer;

	static void MainTask (void * );
	static void PromptTask(void * );

	static void CatchSignal(int sig);

	static const char * s_initTaskName;
	static const char * s_mainTaskName;
	static const char * s_promptTaskName;

	bool ThreadPromptStarted();

	RT_TASK d_main_task;
	RT_TASK d_prompt_task;
	CpgNetworkRTWrapper d_network;
	bool d_prompted;
	ListOfInitializer d_initializers;
};
template <class I>
void XenoCpg::AppendInitializer(){
	Initializer::Ptr i(new I());
	AppendInitializer(i);
}
#endif //XENO_CPG_XENO_CPG_H_
