/*
 * Copyright (c) 2012 Alexandre Tuleu
 *
 * This file is part of xeno-cpg.
 *
 *   xeno-cpg is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   xeno-cpg is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with PROJECT.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * \file XenoCpgConfig.cpp
 *
 *  \date Jun 14, 2012
 *  \author Alexandre Tuleu
 */

#include "XenoCpgConfig.h"
#include <libatuleu/options/Parsers.h>
#include <unistd.h>
#include <stdexcept>

ConfigSection::ConfigSection(options::Section::Ptr section)
	: d_section(section){}

ConfigSection::~ConfigSection(){}


CpgSection::CpgSection(options::Section::Ptr base)
	: ConfigSection(base)
	, d_oNetworkFile(base->AddOption<std::string>("network-file","","File for libcpg/codyn."))
	, d_oTimestep(base->AddOption<double>("timestep",0.005,"loop timestep in second >= 0.005"))
	, d_oIterationPerTimestep(base->AddOption<unsigned int>("iteration-per-step",1,"number of time the cpg network is steped in one loop"))
	, d_cpgOverride(base->AddSubSection("override","put here overriding for lib cpg, by using syntax object::property = expression"))
	, d_mapping(base->AddSubSection("mapping","map here actuator and sensor to some properties"))
	, d_inMapping(d_mapping->AddSubSection("input","Map sensor"))
	, d_outMapping(d_mapping->AddSubSection("output","Map outputs"))
{
}

CpgSection::~CpgSection(){}

options::Section::Ptr & CpgSection::CpgOverride(){
	return d_cpgOverride;
}

options::Section::Ptr& CpgSection::InMapping(){
	return d_inMapping;
}

options::Section::Ptr& CpgSection::OutMapping(){
	return d_outMapping;
}

MainSection::MainSection(options::Section::Ptr section)
	: ConfigSection(section)
	, d_oCliFeedbackIteration(section->AddOption<unsigned int>("cli-feedback-iteration",0,"print feedback every x iteration, 0 disable feedback")){

}
MainSection::~MainSection(){

}


XenoCpgConfig::XenoCpgConfig()
	: d_root(options::Section::Ptr(new options::Section("","")))
	, d_cpg(d_root->AddSubSection("cpg","cpg settings"))
	, d_main(d_root->AddSubSection("main","main section")){
}

XenoCpgConfig::~XenoCpgConfig(){
}


void XenoCpgConfig::SetBaseFileName(const std::string& basename){
	d_basename=basename;
}

void XenoCpgConfig::LoadFile(const std::string& filename){
	options::ConfigFileParser p;
	p.Parse(filename,d_root);
}

void XenoCpgConfig::LoadStdFiles(){
	if(d_basename.empty()){
		throw std::runtime_error("You should SetBaseFileName() before !");
	}

	LoadFile("/etc/" + d_basename);
	LoadFile(getenv("HOME") + ("/." + d_basename));
	LoadFile("./" +  d_basename);

}

CpgSection& XenoCpgConfig::Cpg(){
	return d_cpg;
}

MainSection& XenoCpgConfig::Main(){
	return d_main;
}

