/*
 * Copyright (c) 2012 Alexandre Tuleu
 *
 * This file is part of xeno-cpg.
 *
 *   xeno-cpg is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   xeno-cpg is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with PROJECT.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * \file XenoCpgConfig.h
 *
 *  \date Jun 14, 2012
 *  \author Alexandre Tuleu
 */

#ifndef XENO_CPG_XENOCPGCONFIG_H_
#define XENO_CPG_XENOCPGCONFIG_H_


#include <libatuleu/base/Singleton.h>
#include <libatuleu/options/Option.h>


#define ADD_OPTION_IMPLEMENTATION(type,name)\
	private :\
		options::Option< type >::Ptr d_o ## name;\
	public :\
		inline type name () const { return d_o ## name->Value(); }


class ConfigSection {
public :
	ConfigSection(options::Section::Ptr section);
	virtual ~ConfigSection();

	options::Section::Ptr Section();

private:
	options::Section::Ptr d_section;
};



class CpgSection : public ConfigSection {
public :
	CpgSection(options::Section::Ptr section);
	virtual ~CpgSection();

	ADD_OPTION_IMPLEMENTATION(std::string,NetworkFile)
	ADD_OPTION_IMPLEMENTATION(double,Timestep)
	ADD_OPTION_IMPLEMENTATION(unsigned int, IterationPerTimestep)

	options::Section::Ptr & CpgOverride();
	options::Section::Ptr & InMapping();
	options::Section::Ptr & OutMapping();

private :
	options::Section::Ptr d_cpgOverride;
	options::Section::Ptr d_mapping;
	options::Section::Ptr d_inMapping;
	options::Section::Ptr d_outMapping;
};

class MainSection : public ConfigSection {
public :
	MainSection(options::Section::Ptr section);
	virtual ~MainSection();

	ADD_OPTION_IMPLEMENTATION(unsigned int , CliFeedbackIteration);
};

/**
 *
 */
class XenoCpgConfig :public Singleton<XenoCpgConfig> {
public:
	XenoCpgConfig();
	virtual ~XenoCpgConfig();

	void SetBaseFileName(const std::string & basename);

	void LoadFile(const std::string & filename);

	void LoadStdFiles();

	CpgSection & Cpg();
	MainSection & Main();

public :
	std::string d_basename;

	options::Section::Ptr d_root;
	CpgSection d_cpg;
	MainSection d_main;
};

#endif // XENO_CPG_XENOCPGCONFIG_H_
