/* 
 * Copyright (c) 2012 Alexandre Tuleu
 * 
 * This file is part of xeno-cpg.
 * 
 *   xeno-cpg is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 *   xeno-cpg is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License
 *   along with PROJECT.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * \file XenoCpgPrompt.h
 *
 *  \date May 2, 2012
 *  \author Alexandre Tuleu
 */

#ifndef XENO_CPG_XENOCPGPROMPT_H_
#define XENO_CPG_XENOCPGPROMPT_H_

#include <tr1/memory>

#include <native/task.h>

#include <libatuleu/cli/InteractiveInterface.h>
#include <libatuleu/cli/Variable.h>

#include "CpgNetworkRTWrapper.h"
/**
 *
 */
class XenoCpgPrompt : public cli::Interface{
public:
	class PropertyVariable : public cli::VariableBase {
	public :
		typedef std::tr1::shared_ptr<PropertyVariable> Ptr;

		PropertyVariable(CpgPropertyIO::Ptr var);
		virtual ~PropertyVariable();

		void PreGetValueHook() const;
		void PostSetValueHook();

		const std::string & toString() const;
		bool SetFromString(const std::string & val);

		void Increment(double value);
		void Decrement(double value);
	private :
		void SetStringFromValue();

		double d_value;
		std::string d_stringValue;
		CpgPropertyIO::Ptr d_io;
	};


	class IncrementSelected : public cli::KeyCallback {
	public :
		IncrementSelected(PropertyVariable::Ptr pv,
		                  double increment,
		                  bool decrementInstead = false);
		virtual ~IncrementSelected();

		int Call(int key);
	private :
		PropertyVariable::Ptr d_pv;
		double d_increment;
		bool d_decrement;
	};
	XenoCpgPrompt(CpgNetworkRTWrapper & controller);
	virtual ~XenoCpgPrompt();

	void Init();

	PropertyVariable::Ptr PropertyVariableFromName(const std::string & name);

	void Save(const std::string & filename);

	void Select(const std::string & propName,const double & increment);

	void Unselect();

	void Start();

	void Pause();

private :
	static const char IncrementKey = '+';
	static const char DecrementKey = '"';
	typedef std::map<std::string,PropertyVariable::Ptr> PropertyVariableByName;
	CpgNetworkRTWrapper & d_controller;

	PropertyVariableByName d_cpgProperties;
	PropertyVariable::Ptr d_selected;
};

#endif // XENO_CPG_XENOCPGPROMPT_H_
