#include "task.h"
#include <sys/timerfd.h>
#include <stdint.h>
#include "TaskRegister.h"
#include <unistd.h>

int 	rt_task_create (RT_TASK *task, const char *name, int stksize, int prio, int mode){
	task->timer = 0;
	pthread_attr_t * a = &(task->attr);
	pthread_attr_init(a);
	if( mode & T_JOINABLE ) {
		pthread_attr_setdetachstate(a,PTHREAD_CREATE_JOINABLE);
	}
	if(stksize != 0){
		pthread_attr_setstacksize(a,stksize);
	}

	return 0;
}

int 	rt_task_inquire (RT_TASK *task, RT_TASK_INFO *info){
	return pthread_getschedparam(task->thread,&(info->policy),&(info->param));
}

int 	rt_task_delete (RT_TASK *task){
	pthread_attr_destroy(&(task->attr));
	pthread_detach(task->thread);
	return 0;
}

int 	rt_task_start (RT_TASK *task, void (*entry)(void*), void * cookie){
	typedef void*(*good_fct_ptr)(void*);
	pthread_create( &(task->thread) , &(task->attr) , (good_fct_ptr)entry , cookie);
	return 0;
}

int 	rt_task_set_periodic (RT_TASK *task, RTIME idate, RTIME period){
	task->timer = timerfd_create(CLOCK_MONOTONIC,0);
	struct itimerspec spec,oldvalue;
	spec.it_value.tv_sec = 0;
	spec.it_value.tv_nsec = period;
	spec.it_interval.tv_sec = 0;
	spec.it_interval.tv_nsec = period;
	return timerfd_settime(task->timer,0,&spec,&oldvalue);

}

int 	rt_task_join (RT_TASK *task){
	void * status;
	pthread_join(task->thread ,&status);
	return 0;
}

int 	rt_task_wait_period (unsigned long *overruns_r){
	RT_TASK * self = rt_task_self();

	if (self == NULL){
		return -EPERM;
	}

	if(self->timer == 0){
		return -EWOULDBLOCK;
	}

	uint64_t o;

	//block until next timer tick;
	if( read(self->timer,&o,sizeof(uint64_t)) == -1){
		return -errno;
	}
	if(overruns_r != NULL){
		* overruns_r = o;
	}

	return 0;



}

int 	rt_task_set_mode (int clrmask, int setmask, int *mode_r){
	return 0;
}

int 	rt_task_sleep (RTIME delay){
	usleep(delay);
	return 0;
}
